Learn R Programming

betapart (version 1.2)

betapart.core: Core calculations of betapart

Description

Computes the basic quantities needed for computing the multiple-site beta diversity measures and pairwise dissimilarity matrices.

Usage

betapart.core(x)

Arguments

x
data frame, where rows are sites and columns are species

Value

  • The function returns an object of class betapart with the following elements:
  • sumSithe sum of the species richness values of all sites
  • Stthe total richness in the dataset
  • athe multiple-site analog of the shared species term
  • shareda matrix containing the number of species shared between pairs of sites
  • not.shareda matrix containing the number of species not shared between pairs of sites: b, c
  • sum.not.shareda matrix containing the total number of species not shared between pairs of sites: b+c
  • max.not.shareda matrix containing the total maximum number of species not shared between pairs of sites: max(b,c)
  • min.not.shareda matrix containing the total minimum number of species not shared between pairs of sites: min(b,c)

encoding

utf8

References

Baselga, A. 2010. Partitioning the turnover and nestedness components of beta diversity. Global Ecology and Biogeography 19:134-143 Baselga, A. 2012. The relationship between species replacement, dissimilarity derived from nestedness, and nestedness. Global Ecology and Biogeography 21, 1223-1232

See Also

beta.multi, beta.pair, beta.sample, beta.temp,

Examples

Run this code
data(ceram.s)
ceram.core.s<-betapart.core(ceram.s)
ceram.dist.jac<-beta.pair(ceram.core.s, index.family="jac")
ceram.dist.sor<-beta.pair(ceram.core.s, index.family="sor")
ceram.multi.jac<-beta.multi(ceram.core.s, index.family="jac")
ceram.multi.sor<-beta.multi(ceram.core.s, index.family="sor")

Run the code above in your browser using DataLab